@@ -262,14 +262,45 @@ static inline bool is_global(enum label_type type)
     return type == LBL_GLOBAL || type == LBL_COMMON;
 }
 
+enum mangle_index {
+    LM_LPREFIX,                 /* Local variable prefix */
+    LM_LSUFFIX,                 /* Local variable suffix */
+    LM_GPREFIX,                 /* Global variable prefix */
+    LM_GSUFFIX                  /* GLobal variable suffix */
+};
+
 static const char *mangle_strings[] = {"", "", "", ""};
 static bool mangle_string_set[ARRAY_SIZE(mangle_strings)];
 
 /*
  * Set a prefix or suffix
  */
-void set_label_mangle(enum mangle_index which, const char *what)
+void set_label_mangle(enum directive how, const char *what)
 {
+    enum mangle_index which;
+
+    switch (how) {
+    case D_PREFIX:
+    case D_GPREFIX:
+        which = LM_GPREFIX;
+        break;
+    case D_SUFFIX:
+    case D_GSUFFIX:
+    case D_POSTFIX:
+    case D_GPOSTFIX:
+        which = LM_GSUFFIX;
+        break;
+    case D_LPREFIX:
+        which = LM_LPREFIX;
+        break;
+    case D_LSUFFIX:
+    case D_LPOSTFIX:
+        which = LM_LSUFFIX;
+        break;
+    default:
+        return;
+    }
+
     if (mangle_string_set[which])
         return;                 /* Once set, do not change */
 
@@ -495,12 +526,6 @@ void define_label(const char *label, int32_t segment,
             nasm_nonfatal("label `%s' inconsistently redefined", lptr->defn.label);
             noteflags = ERR_NONFATAL|ERR_HERE|ERR_NO_SEVERITY;
         } else {
-            /*!
-             *!label-redef [off] label redefined to an identical value
-             *!  warns if a label is defined more than once, but the
-             *!  value is identical. It is an unconditional error to
-             *!  define the same label more than once to \e{different} values.
-             */
             nasm_warn(WARN_LABEL_REDEF,
                        "info: label `%s' redefined to an identical value", lptr->defn.label);
             noteflags = ERR_WARNING|ERR_HERE|ERR_NO_SEVERITY|WARN_LABEL_REDEF;
@@ -511,19 +536,7 @@ void define_label(const char *label, int32_t segment,
         nasm_error(noteflags, "info: label `%s' originally defined", lptr->defn.label);
         src_set(saved_line, saved_fname);
     } else if (changed && pass_final() && lptr->defn.type != LBL_SPECIAL) {
-        /*!
-         *!label-redef-late [err] label (re)defined during code generation
-         *!  the value of a label changed during the final, code-generation
-         *!  pass. This may be the result of strange use of the
-         *!  preprocessor. This is very likely to produce incorrect code and
-         *!  may end up being an unconditional error in a future
-         *!  version of NASM.
-         *
-         * WARN_LABEL_LATE defaults to an error, as this should never
-         * actually happen.  Just in case this is a backwards
-         * compatibility problem, still make it a warning so that the
-         * user can suppress or demote it.
-         *
+        /*
          * Note: As a special case, LBL_SPECIAL symbols are allowed
          * to be changed even during the last pass.
          */
