@@ -300,14 +300,6 @@ static void mref_set_optype(operand *op)
                     flag = IP_REL;
                 if (!(globl.reldef & op->eaflags)) {
                     static int64_t pass_last_seen;
-                    /*!
-                     *!implicit-abs-deprecated [on] implicit DEFAULT ABS is deprecated
-                     *!
-                     *!  warns that in a subsequent version of NASM, the 64-bit default
-                     *!  addressing form is likely to change from \c{DEFAULT ABS} to
-                     *!  \c{DEFAULT REL}. If absolute addressing is indeed intended, it is
-                     *!  strongly recommended to specify \c{DEFAULT ABS} explicitly.
-                     */
                     if (pass_count() != pass_last_seen) {
                         nasm_warn(WARN_IMPLICIT_ABS_DEPRECATED,
                                   "implicit DEFAULT ABS is deprecated");
@@ -754,6 +746,7 @@ restart_parse:
 
     if (i == TOKEN_ID || (insn_is_label && i == TOKEN_INSN)) {
         /* there's a label here */
+        struct tokenval label = tokval;
         first = false;
         result->label = tokval.t_charptr;
         i = stdscan(NULL, &tokval);
@@ -761,15 +754,9 @@ restart_parse:
         if (i == ':') {         /* skip over the optional colon */
             i = stdscan(NULL, &tokval);
         } else if (i == 0) {
-            /*!
-             *!label-orphan [on] labels alone on lines without trailing \c{:}
-             *!=orphan-labels
-             *!  warns about source lines which contain no instruction but define
-             *!  a label without a trailing colon. This is most likely indicative
-             *!  of a typo, but is technically correct NASM syntax (see \k{syntax}.)
-             */
-            nasm_warn(WARN_LABEL_ORPHAN ,
-                      "label alone on a line without a colon might be in error");
+            nasm_warn(WARN_LABEL_ORPHAN,
+                      "label `%*s' alone on a line without a colon might be in error",
+                      (int)label.t_len, label.t_start);
         }
         if (i != TOKEN_INSN || tokval.t_integer != I_EQU) {
             /*
@@ -909,13 +896,6 @@ restart_parse:
             /* DB et al */
             result->operands = oper_num;
             if (oper_num == 0)
-                /*!
-                 *!db-empty [on] no operand for data declaration
-                 *!  warns about a \c{D}\e{x} declaration
-                 *!  with no operands, producing no output.
-                 *!  This is permitted, but often indicative of an error.
-                 *!  See \k{db}.
-                 */
                 nasm_warn(WARN_DB_EMPTY, "no operand for data declaration");
         }
         return result;
@@ -984,8 +964,10 @@ restart_parse:
         while (i == TOKEN_SPECIAL || i == TOKEN_SIZE) {
             switch (tokval.t_integer) {
             case S_BYTE:
-                if (!setsize)   /* we want to use only the first */
+                if (!setsize) {   /* we want to use only the first */
+                    result->opt |= OPTIM_NO_Jcc_RELAX | OPTIM_NO_JMP_RELAX;
                     op->type |= BITS8;
+                }
                 setsize = 1;
                 break;
             case S_WORD:
@@ -1034,9 +1016,12 @@ restart_parse:
                 op->type |= FAR;
                 break;
             case S_NEAR:
+                /* This is not legacy behavior, even if it perhaps should be */
+                /* result->opt |= OPTIM_NO_Jcc_RELAX | OPTIM_NO_JMP_RELAX; */
                 op->type |= NEAR;
                 break;
             case S_SHORT:
+                result->opt |= OPTIM_NO_Jcc_RELAX | OPTIM_NO_JMP_RELAX;
                 op->type |= SHORT;
                 break;
             case S_ABS:
@@ -1224,7 +1209,8 @@ restart_parse:
                 i = tokval.t_type;
             }
             if (!recover && i != 0 && i != ',') {
-                nasm_nonfatal("comma, decorator or end of line expected, got %d", i);
+                nasm_nonfatal("comma, decorator or end of line expected, got `%*s'",
+                              (int)tokval.t_len, tokval.t_start);
                 recover = true;
             }
         } else {                /* immediate operand */
@@ -1397,21 +1383,6 @@ restart_parse:
                     if (!opsize) {
                         op->type |= rs; /* For non-size-specific registers, permit size override */
                     } else if (opsize != rs) {
-                        /*!
-                         *!regsize [on] register size specification ignored
-                         *!
-                         *!  warns about a register with implicit size (such as \c{EAX}, which is always 32 bits)
-                         *!  been given an explicit size specification which is inconsistent with the size
-                         *!  of the named register, e.g. \c{WORD EAX}. \c{DWORD EAX} or \c{WORD AX} are
-                         *!  permitted, and do not trigger this warning. Some registers which \e{do not} imply
-                         *!  a specific size, such as \c{K0}, may need this specification unless the instruction
-                         *!  itself implies the instruction size:
-                         *!-
-                         *!  \c      KMOVW K0,[foo]          ; Permitted, KMOVW implies 16 bits
-                         *!  \c      KMOV  WORD K0,[foo]     ; Permitted, WORD K0 specifies instruction size
-                         *!  \c      KMOV  K0,WORD [foo]     ; Permitted, WORD [foo] specifies instruction size
-                         *!  \c      KMOV  K0,[foo]          ; Not permitted, instruction size ambiguous
-                         */
                         nasm_warn(WARN_REGSIZE, "invalid register size specification ignored");
                     }
                 }
