@@ -318,15 +318,6 @@ static int stdscan_token(struct tokenval *tv)
             int token_type = nasm_token_hash(tv->t_charptr, tv);
 
             if (unlikely(tv->t_flag & TFLAG_WARN)) {
-                /*! ptr [on] non-NASM keyword used in other assemblers
-                 *!  warns about keywords used in other assemblers that
-                 *!  might indicate a mistake in the source code.
-                 *!  Currently only the MASM \c{PTR} keyword is
-                 *!  recognized. If (limited) MASM compatibility is
-                 *!  desired, the \c{%use masm} macro package is
-                 *!  available, see \k{pkg_masm}; however, carefully note
-                 *!  the caveats listed.
-                 */
                 nasm_warn(WARN_PTR, "`%s' is not a NASM keyword",
                           tv->t_charptr);
             }
