@@ -1,36 +1,6 @@
 #!/usr/bin/perl
-## --------------------------------------------------------------------------
-##
-##   Copyright 1996-2018 The NASM Authors - All Rights Reserved
-##   See the file AUTHORS included with the NASM distribution for
-##   the specific copyright holders.
-##
-##   Redistribution and use in source and binary forms, with or without
-##   modification, are permitted provided that the following
-##   conditions are met:
-##
-##   * Redistributions of source code must retain the above copyright
-##     notice, this list of conditions and the following disclaimer.
-##   * Redistributions in binary form must reproduce the above
-##     copyright notice, this list of conditions and the following
-##     disclaimer in the documentation and/or other materials provided
-##     with the distribution.
-##
-##     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-##     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-##     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
-##     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-##     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-##     CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-##     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
-##     NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-##     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-##     HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-##     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-##     OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
-##     EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-##
-## --------------------------------------------------------------------------
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 1996-2025 The NASM Authors - All Rights Reserved
 
 
 # Read the source-form of the NASM manual and generate the various
@@ -86,6 +56,9 @@
 #   the \W prefix is ignored except in HTML; in HTML the last part
 #   becomes a hyperlink to the first part.
 #
+# Web URL \w{http://foobar/}
+#   equivalent to \W{http://foobar}\c{http://foobar/}.
+#
 # Literals \{ \} \\
 #   In case it's necessary, they expand to the real versions.
 #
@@ -110,10 +83,20 @@
 #   operator was applied to
 #
 # Index alias
-# \IA{foobar}{bazquux}
+# \IA{foobar}{bazquux} [tidy...]
 #   aliases one index tag (as might be supplied to \i or \I) to
 #   another, so that \I{foobar} has the effect of \I{bazquux}, and
-#   \i{foobar} has the effect of \I{bazquux}foobar
+#   \i{foobar} has the effect of \I{bazquux}foobar.
+#
+#  If a "tidy" string is provided, it also performs
+#   the function of \IR.
+#
+# Index copy
+# \IC{foobar}{bazquux} [tidy...]
+#   similar to \IA, but duplicates all the index entries from
+#   "foobar" onto the index entry "bazquux", as if every \i{foobar}
+#   or \I{foobar}, or its aliases defined by \IA, was immediately followed by
+#   \I{bazquux}.
 #
 # Metadata/macros
 # \M{key}{something}
@@ -123,8 +106,8 @@
 #   insert the {something} string associated with metadata {key}
 #
 # Include subfile
-# \&{filename}
-#  Includes filename. Recursion is allowed.
+# \& filename
+#   includes filename. Recursion is allowed. Must be on a separate line.
 #
 
 use File::Spec;
@@ -132,10 +115,15 @@ use File::Spec;
 @include_path = ();
 $out_path = File::Spec->curdir();
 
+my %srcfiles;			# For dependencies
+my $depend_path;
+
 while ($ARGV[0] =~ /^-/) {
     my $opt = shift @ARGV;
     if ($opt eq '-d') {
 	$diag = 1;
+    } elsif ($opt =~ /^\-M(.*)$/) {
+	$depend_path = $1;
     } elsif ($opt =~ /^\-[Ii](.*)$/) {
 	push(@include_path, $1);
     } elsif ($opt =~ /^\-[Oo](.*)$/) {
@@ -156,7 +144,6 @@ $tstruct_last[$tstruct_level{$tstruct_previtem}] = $tstruct_previtem;
 $MAXLEVEL = 10;  # really 3, but play safe ;-)
 
 # Read the file; pass a paragraph at a time to the paragraph processor.
-print "Reading input...";
 $pname = [];
 @pnames = @pflags = ();
 $para = undef;
@@ -164,45 +151,83 @@ foreach $file (@files) {
   &include($file);
 }
 &got_para($para);
-print "done.\n";
+print "$outfile: done.\n";
 
 # Now we've read in the entire document and we know what all the
 # heading keywords refer to. Go through and fix up the \k references.
-print "Fixing up cross-references...";
+print "$outfile: Fixing up cross-references...\n";
 &fixup_xrefs;
-print "done.\n";
 
 # Sort the index tags, according to the slightly odd order I've decided on.
-print "Sorting index tags...";
+print "$outfile: sorting index tags...\n";
 &indexsort;
-print "done.\n";
 
 # Make output directory if necessary
 mkdir($out_path);
 
 if ($diag) {
-  print "Writing index-diagnostic file...";
+  print "$outfile: writing index-diagnostic file...\n";
   &indexdiag;
-  print "done.\n";
 }
 
 # OK. Write out the various output files.
+my $outfile;
 if ($out_format eq 'txt') {
-    print "Producing text output: ";
+    $outfile = 'nasmdoc.txt';
+    print "$outfile: producing text output...\n";
     &write_txt;
-    print "done.\n";
 } elsif ($out_format eq 'html') {
-    print "Producing HTML output: ";
+    $outfile = 'nasm00.html';
+    print "$outfile: producing HTML output...\n";
     &write_html;
-    print "done.\n";
 } elsif ($out_format eq 'dip') {
-    print "Producing Documentation Intermediate Paragraphs: ";
+    $outfile = 'nasmdoc.dip';
+    print "$outfile: producing Documentation Intermediate Paragraphs...\n";
     &write_dip;
-    print "done.\n";
 } else {
     die "$0: unknown output format: $out_format\n";
 }
 
+if (defined($depend_path)) {
+    # Write dependencies
+    print "$outfile: writing dependencies\n";
+    open(my $dep, '>', $depend_path)
+	or die "$outfile: $depend_path: $!\n";
+
+    if ($out_path ne File::Spec->curdir()) {
+	$outfile = File::Spec->catfile($out_path, $outfile);
+    }
+
+    my $o = $outfile.' :';
+    my $ol = length($o);
+    foreach my $sf (sort(keys(%srcfiles))) {
+	my $l = length($sf);
+	if ($l + $ol > 77) {
+	    print $dep $o, " \\\n";
+	    $o = '';
+	    $ol = 0;
+	}
+	$o .= ' '.$sf;
+	$ol += $l+1;
+    }
+    print $dep $o, "\n\n";
+    close($dep);
+}
+
+print "$outfile: done.\n";
+
+sub refpush(\$@) {
+    my $ref = shift(@_);
+    $$ref = [] unless (defined($$ref));
+    push(@$$ref, @_);
+    return $$ref;
+}
+sub reflist($) {
+    my($ref) = @_;
+    return () unless (defined($ref));
+    return @$ref;
+}
+
 sub untabify($) {
   my($s) = @_;
   my $o = '';
@@ -234,7 +259,7 @@ sub read_line {
 }
 sub get_para($_) {
   chomp;
-  if (!/\S/ || /^\\(IA|IR|M)/) { # special case: \IA \IR \M imply new-paragraph
+  if (!/\S/ || /^\\(I[ARC]|M)/) { # special case: \I[ARC] \M imply new-paragraph
     &got_para($para);
     $para = undef;
   }
@@ -248,17 +273,22 @@ sub include {
   my $F;
 
   if ($name eq '-') {
-    open($F, '<-');		# stdin
+      open($F, '<&', \*STDIN);		# stdin
+      print "$outfile: reading stdin...\n";
   } else {
     my $found = 0;
     foreach my $idir ( File::Spec->curdir, @include_path ) {
 	my $fpath = File::Spec->catfile($idir, $name);
-      if (open($F, '<', $fpath)) {
-	$found = 1;
-	last;
-      }
+	if (open($F, '<', $fpath)) {
+	    # Assume that make uses VPATH for the input search path,
+	    # and so dependencies should not include the search directory.
+	    $srcfiles{$name}++;
+	    $found = 1;
+	    print "$outfile: reading $fpath...\n";
+	    last;
+	}
     }
-    die "Cannot open $name: $!\n" unless ($found);
+    die "$0:$outfile: Cannot open $name: $!\n" unless ($found);
   }
   while (defined($_ = <$F>)) {
      &read_line($_);
@@ -305,7 +335,7 @@ sub got_para {
     $snum = 0;
     $xref = "chapter-$cnum";
     $pflags = "chap $cnum :$xref";
-    die "badly formatted chapter heading: $_\n" if !/^\\C\{([^\}]*)\}\s*(.*)$/;
+    die "$outfile: badly formatted chapter heading: $_\n" if !/^\\C\{([^\}]*)\}\s*(.*)$/;
     $refs{$1} = "chapter $cnum";
     $node = "Chapter $cnum";
     &add_item($node, 1, $para);
@@ -322,7 +352,7 @@ sub got_para {
     $snum = 0;
     $xref = "appendix-$cnum";
     $pflags = "appn $cnum :$xref";
-    die "badly formatted appendix heading: $_\n" if !/^\\A\{([^\}]*)}\s*(.*)$/;
+    die "$outfile: badly formatted appendix heading: $_\n" if !/^\\A\{([^\}]*)}\s*(.*)$/;
     $refs{$1} = "appendix $cnum";
     $node = "Appendix $cnum";
     &add_item($node, 1, $para);
@@ -336,7 +366,7 @@ sub got_para {
     $snum = 0;
     $xref = "section-$cnum.$hnum";
     $pflags = "head $cnum.$hnum :$xref";
-    die "badly formatted heading: $_\n" if !/^\\[HP]\{([^\}]*)\}\s*(.*)$/;
+    die "$outfile: badly formatted heading: $_\n" if !/^\\[HP]\{([^\}]*)\}\s*(.*)$/;
     $refs{$1} = "section $cnum.$hnum";
     $node = "Section $cnum.$hnum";
     &add_item($node, 2, $para);
@@ -349,7 +379,7 @@ sub got_para {
     $snum++;
     $xref = "section-$cnum.$hnum.$snum";
     $pflags = "subh $cnum.$hnum.$snum :$xref";
-    die "badly formatted subheading: $_\n" if !/^\\S\{([^\}]*)\}\s*(.*)$/;
+    die "$outfile: badly formatted subheading: $_\n" if !/^\\S\{([^\}]*)\}\s*(.*)$/;
     $refs{$1} = "section $cnum.$hnum.$snum";
     $node = "Section $cnum.$hnum.$snum";
     &add_item($node, 3, $para);
@@ -359,18 +389,30 @@ sub got_para {
     # the standard word-by-word code will happen next
   } elsif (/^\\IR/) {
     # An index-rewrite.
-    die "badly formatted index rewrite: $_\n" if !/^\\IR\{([^\}]*)\}\s*(.*)$/;
+    die "$outfile: badly formatted index rewrite: $_\n" if !/^\\IR\{([^\}]*)\}\s*(.+?)\s*$/;
     $irewrite = $1;
     $_ = $2;
     # the standard word-by-word code will happen next
-  } elsif (/^\\IA/) {
-    # An index-alias.
-    die "badly formatted index alias: $_\n" if !/^\\IA\{([^\}]*)}\{([^\}]*)\}\s*$/;
-    $idxalias{$1} = $2;
-    return; # avoid word-by-word code
+  } elsif (/^\\I([AC])/) {
+      # An index alias or copy
+      my $what = $1 eq 'C' ? 'copy' : 'alias';
+      die "$outfile: badly formatted index $what: $_\n"
+	  if !/^\\I[AC]\{([^\}]*)}\{([^\}]*)\}\s*(.*?)\s*$/;
+      my $from = $1;
+      my $to   = $2;
+      my $tidy = $3;
+      if ($what eq 'copy') {
+	  refpush($idxcopy{$from}, $to);
+      } else {
+	  $idxalias{$from} = $to;
+      }
+      return if ($tidy eq '');	# No rewrite, skip word by word code
+
+      $irewrite = $to;
+      $_ = $tidy;
   } elsif (/^\\M/) {
     # Metadata
-    die "badly formed metadata: $_\n" if !/^\\M\{([^\}]*)}\{([^\}]*)\}\s*$/;
+    die "$outfile: badly formed metadata: $_\n" if !/^\\M\{([^\}]*)}\{([^\}]*)\}\s*$/;
     $metadata{$1} = $2;
     return; # avoid word-by-word code
   } elsif (/^\\([b\>q])/) {
@@ -422,7 +464,7 @@ sub got_para {
       $qindex = 1 if $1 eq "\\I";
       $indexing = 1, s/^\\[iI]// if $1;
       s/^\\c//;
-      die "badly formatted \\c: \\c$_\n" if !/\{(([^\\}]|\\.)*)\}(.*)$/;
+      die "$outfile: badly formatted \\c: \\c$_\n" if !/\{(([^\\}]|\\.)*)\}(.*)$/;
       $w = $1;
       $_ = $3;
       $w =~ s/\\\{/\{/g;
@@ -438,7 +480,7 @@ sub got_para {
       $indexing = 1, $type = "\\i" if $1;
       $emph = 1, $type = "\\e" if $2;
       s/^(\\[iI])?(\\e?)//;
-      die "badly formatted $type: $type$_\n" if !/\{(([^\\}]|\\.)*)\}(.*)$/;
+      die "$outfile: badly formatted $type: $type$_\n" if !/\{(([^\\}]|\\.)*)\}(.*)$/;
       $w = $1;
       $_ = $3;
       $w =~ s/\\\{/\{/g;
@@ -471,28 +513,37 @@ sub got_para {
       $t = "k ";
       $t = "kK" if /^\\K/;
       s/^\\[kK]//;
-      die "badly formatted \\k: \\k$_\n" if !/\{([^\}]*)\}(.*)$/;
+      die "$outfile: badly formatted \\k: \\k$_\n" if !/\{([^\}]*)\}(.*)$/;
       $_ = $2;
       push @$para,"$t$1";
-    } elsif (/^\\W/) {
-      s/^\\W//;
-      die "badly formatted \\W: \\W$_\n"
-          if !/\{([^\}]*)\}(\\i)?(\\c)?\{(([^\\}]|\\.)*)\}(.*)$/;
-      $l = $1;
-      $w = $4;
-      $_ = $6;
-      $t = "w ";
-      $t = "wc" if $3 eq "\\c";
-      $indexing = 1 if $2;
-      $w =~ s/\\\{/\{/g;
-      $w =~ s/\\\}/\}/g;
-      $w =~ s/\\-/-/g;
-      $w =~ s/\\\\/\\/g;
-      push(@$para, addidx($node, $w, "c $w")) if $indexing;
-      push(@$para, "$t<$l>$w");
+    } elsif (/^\\[Ww]/) {
+	if (/^\\w/) {
+	    die "$outfile: badly formatted \\w: $_\n"
+		if !/^\\w(\\i)?\{([^\\}]*)\}(.*)$/;
+	    $l = $2;
+	    $w = $2;
+	    $indexing = $1;
+	    $c = 1;
+	    $_ = $3;
+	} else {
+	    die "$outfile: badly formatted \\W: $_\n"
+		if !/^\\W\{([^\\}]*)\}(\\i)?(\\c)?\{(([^\\}]|\\.)*)\}(.*)$/;
+	    $l = $1;
+	    $w = $4;
+	    $_ = $6;
+	    $indexing = $2;
+	    $c = $3;
+	}
+	$t = $c ? 'wc' : 'w ';
+	$w =~ s/\\\{/\{/g;
+	$w =~ s/\\\}/\}/g;
+	$w =~ s/\\-/-/g;
+	$w =~ s/\\\\/\\/g;
+	push(@$para, addidx($node, $w, "c $w")) if $indexing;
+	push(@$para, "$t<$l>$w");
     } else {
-      die "what the hell? $_\n" if !/^(([^\s\\\-]|\\[\\{}\-])*-?)(.*)$/;
-      die "painful death! $_\n" if !length $1;
+      die "$outfile: what the hell? $_\n" if !/^(([^\s\\\-]|\\[\\{}\-])*-?)(.*)$/;
+      die "$outfile: painful death! $_\n" if !length $1;
       $w = $1;
       $_ = $3;
       $w =~ s/\\\{/\{/g;
@@ -516,21 +567,35 @@ sub got_para {
   }
 }
 
+sub indexalias($) {
+    my($text) = @_;
+    my $a = $idxalias{$text};
+    return defined($a) ? $a : $text;
+}
+
 sub addidx($$@) {
   my($node, $text, @ientry) = @_;
-  $text = $idxalias{$text} || $text;
-  if (!exists($idxmap{$text})) {
-      $idxmap{$text} = [@ientry];
-      $idxdup{$text} = [$text];
-  } elsif (!defined($node)) {
-      my $dummy = sprintf('%s    #%05d', $text, $#{$idxdup{$text}} + 2);
-      $idxmap{$dummy} = [@ientry];
-      push(@{$idxdup{$text}}, $dummy);
+
+  my $ta = indexalias($text);
+
+  my @out;
+  foreach my $t ($text, reflist($idxcopy{$text})) {
+      $t = indexalias($t);
+      if (!exists($idxmap{$t})) {
+	  $idxmap{$t} = [@ientry];
+	  $idxdup{$t} = [$t];
+      } elsif (!defined($node)) {
+	  my $dupentry = sprintf('%s    #%05d', $t, $#{$idxdup{$t}} + 2);
+	  $idxmap{$dummy} = [@ientry];
+	  refpush($idxdup{$t}, $dummy);
+      }
+
+      if (defined($node)) {
+	  push(@out, map { $idxnodes{$node,$_} = 1; "i $_" } @{$idxdup{$t}});
+      }
   }
 
-  return undef if (!defined($node));
-
-  return map { $idxnodes{$node,$_} = 1; "i $_" } @{$idxdup{$text}};
+  return @out;
 }
 
 sub indexsort {
@@ -621,7 +686,7 @@ sub fixup_xrefs {
         $caps = ($k =~ /^kK/);
 	$k = substr($k,2);
         $repl = $refs{$k};
-	die "undefined keyword `$k'\n" unless $repl;
+	die "$outfile: undefined keyword `$k'\n" unless $repl;
 	substr($repl,0,1) =~ tr/a-z/A-Z/ if $caps;
 	@repl = ();
 	push @repl,"x $xrefs{$k}";
@@ -765,75 +830,190 @@ sub word_txt {
   } elsif ($wmajt eq "x" || $wmajt eq "i") {
     return "\001";
   } else {
-    die "panic in word_txt: $wtype$w\n";
+    die "$outfile: panic in word_txt: $wtype$w\n";
   }
 }
 
 sub html_filename($) {
     my($node) = @_;
 
-    (my $number = lc($xrefnodes{$node})) =~ s/.*-//;
-    my $fname="nasmdocx.html";
-    substr($fname,8 - length $number, length $number) = $number;
-    return $fname;
+    $node = lc($node);
+    if ($node eq 'contents') {
+	return 'nasm00.html';
+    } elsif ($node eq 'index') {
+	return 'nasmix.html';
+    } else {
+	$node =~ /^(\w+)(?:[ -](\w+))?/;
+	my $type = $1;
+	my $number = $2;
+	if ($type eq 'chapter') {
+	    return sprintf 'nasm%02d.html', $number;
+	} else {
+	    # Appendix
+	    return "nasma${number}.html";
+	}
+    }
+}
+
+my $html_fh;
+sub html_openfile($) {
+    my($node) = @_;
+    die if (defined($html_fh));
+    my $filename = html_filename($node);
+    my $pathname = File::Spec->catfile($out_path, $filename);
+    open($html_fh, '>', $pathname)
+	or die "$0: $pathname: $!\n";
+    select $html_fh;
+    return $filename;
+}
+sub html_closefile() {
+    if (defined($html_fh)) {
+        select STDOUT;
+        close($html_fh);
+	undef $html_fh;
+    }
+}
+
+my @html_navbar;
+
+sub html_navbar_generate() {
+    @html_navbar = (['Contents', html_filename('contents'), 'toc']);
+    my $ctype;
+    for ($node = $tstruct_next{'Top'}; $node; $node = $tstruct_next{$node}) {
+	my $plevel = $tstruct_level{$node};
+
+	next unless ($plevel == 1);
+	next unless ($node =~ /^(\w+) ([\w\.]+)?/);
+	my $nctype = $1;
+	my $nname  = $2;
+
+	if ($nctype ne $ctype) {
+	    $nname = "$nctype\&ensp;$nname";
+	    $ctype = $nctype;
+	}
+	push(@html_navbar, [$nname, html_filename($node), lc($ctype)]);
+    }
+    push(@html_navbar, ['Index', html_filename('Index'), 'index']);
+}
+
+# Open an HTML file and write common preamble code
+sub html_preamble($) {
+    my($node) = @_;
+    my $filename = html_openfile($node);
+    $node =~ /^(\w+)(?:[ -](\w+))?/;
+    my $nodetype = lc($1);
+    my $nodenum  = $2;
+    $nodetype = 'toc' if ($nodetype eq 'contents');
+
+    print "<!DOCTYPE html>\n";
+    print "<head>\n";
+    print "<meta charset=\"UTF-8\" />\n";
+    print "<title>", $metadata{'title'}, "</title>\n";
+    print "<link href=\"nasmdoc.css\" rel=\"stylesheet\" type=\"text/css\" />\n";
+    print "<link href=\"local.css\" rel=\"stylesheet\" type=\"text/css\" />\n";
+    print "</head>\n";
+    print "<body>\n";
+
+    # Navigation bar
+    print "<div class=\"header\">\n";
+    # Stray whitespace inside the <nav> tag cause problems; handle by
+    # putting line breaks inside HTML comments. HACK!
+    print "<nav class=\"navbar\" role=\"navigation\"><ul><!--\n";
+    foreach my $nv (@html_navbar) {
+	my $is_this = '';
+	$is_this = ' navthis' if ($nv->[1] eq $filename);
+	printf "--><li class=\"nav%s%s\"><a href=\"%s\">%s</a></li><!--\n",
+	    $nv->[2], $is_this, $nv->[1], $nv->[0];
+    }
+    print "--></ul></nav>\n";
+
+    print "<div class=\"title\">\n";
+    print "<h1>", $metadata{'title'}, "</h1>\n";
+    print '<h2>', $metadata{'subtitle'}, "</h2>\n";
+    print "</div>\n";
+    print "</div>\n";
+    print "<div class=\"contents $nodetype\">\n";
+}
+
+# Write common postable code and close an HTML file
+sub html_postamble {
+    return unless (defined($html_fh));
+    print "</div>\n</body>\n</html>\n";
+    html_closefile();
 }
 
 sub write_html {
-  # This is called from the top level, so I won't bother using
-  # my or local.
+    # Create the navbar list
+    html_navbar_generate();
 
-  # Write contents file. Just the preamble, then a menu of links to the
+    # Write contents file. Just the preamble, then a menu of links to the
   # separate chapter files and the nodes therein.
   print "writing contents file...";
-  open TEXT, '>', File::Spec->catfile($out_path, 'nasmdoc0.html');
-  select TEXT;
-  undef $html_nav_last;
-  $html_nav_next = $tstruct_next{'Top'};
-  &html_preamble(0);
-  print "<p>This manual documents NASM, the Netwide Assembler: an assembler\n";
-  print "targeting the Intel x86 series of processors, with portable source.\n</p>";
-  print "<div class=\"toc\">\n";
+  html_preamble('contents');
+  print "<h2>Table of Contents</h2>\n";
   $level = 0;
-  for ($node = $tstruct_next{'Top'}; $node; $node = $tstruct_next{$node}) {
-      my $lastlevel = $level;
-      while ($tstruct_level{$node} < $level) {
-	  print "</li>\n</ol>\n";
-	  $level--;
-      }
-      while ($tstruct_level{$node} > $level) {
-	  print "<ol class=\"toc", ++$level, "\">\n";
-      }
-      if ($lastlevel >= $level) {
+  $ollevel = 0;
+
+  sub toc_close_tags($) {
+      my($plevel) = @_;
+      while ($plevel < $level) {
 	  print "</li>\n";
+	  if ($level-- <= $ollevel) {
+	      print "</ol>\n";
+	      $ollevel--;
+	  }
       }
-      $level = $tstruct_level{$node};
+  }
+
+  undef $ctype;			# Chapter or Appendix
+  for ($node = $tstruct_next{'Top'}; $node; $node = $tstruct_next{$node}) {
+      my $plevel = $tstruct_level{$node};
+      my @pnn = split(/[ \.]/, $node);
+      (my $nname = $node) =~ s/^.*?\s+//;
+      my $nnum = $pnn[-1] + 0 || ord($pnn[-1]) - ord('A') + 1;
+      my $nctype = lc($pnn[0]);
+
+      toc_close_tags($plevel);
+      if ($plevel < 2 && $nctype ne $ctype) {
+	  toc_close_tags(0);
+	  my $plural = $nctype;
+	  $plural =~ s/^(.)/\U$1/;
+	  $plural =~ s/ix$/ice/; # ix -> ice + s -> ices
+	  $plural .= 's';
+	  print "<h3 class=\"tocheading $nctype\">$plural</h3>\n";
+	  $ctype = $nctype;
+      }
+
+      while ($plevel > $level) {
+	  $level++;
+	  my $cclass = ($level == 1) ? " $ctype" : '';
+	  print "<ol class=\"toc${level}${cclass}\"", ">\n";
+	  $ollevel = $level;
+      }
+
       if ($level == 1) {
 	  $link = $fname = html_filename($node);
       } else {
 	  # Use the preceding filename plus a marker point.
 	  $link = $fname . "#$xrefnodes{$node}";
       }
-      $pname = $tstruct_pname{$node};
-      $title = plist_to_html(@$pname);
-      print "<li class=\"toc${level}\">\n";
-      print "<span class=\"node\">$node: </span><a href=\"$link\">$title</a>\n";
+
+      my $pname = $tstruct_pname{$node};
+      my $title = plist_to_html(@$pname);
+      printf "<li value=\"%d\" data-name=\"%s\">\n", $nnum, $nname;
+      # The $node span is obsolete and is only included for now to avoid
+      # breaking any existing local.css files.
+      printf "<a href=\"%s\"><span class=\"node\">%s: </span>%s</a>\n",
+	  $link, $node, $title;
   }
-  while ($level--) {
-      print "</li>\n</ol>\n";
-  }
-  print "</div>\n";
-  print "</body>\n";
-  print "</html>\n";
-  select STDOUT;
-  close TEXT;
+  toc_close_tags(0);
+  html_postamble();
 
   # Open a null file, to ensure output (eg random &html_jumppoints calls)
   # goes _somewhere_.
   print "writing chapter files...";
-  open TEXT, '>', File::Spec->devnull();
-  select TEXT;
-  undef $html_nav_last;
-  undef $html_nav_next;
+  # open TEXT, '>', File::Spec->devnull();
+  # select TEXT;
 
   $in_list = 0;
   $in_bquo = 0;
@@ -850,46 +1030,27 @@ sub write_html {
 
     $endtag = '';
 
-    if ($ptype eq "chap") {
-      # Chapter heading. Begin a new file.
-      $pflags =~ /chap (.*) :(.*)/;
-      $title = "Chapter $1: ";
+    if ($ptype eq 'chap' || $ptype eq 'appn') {
+      # Chapter/appendix heading. Begin a new file.
+      $pflags =~ /^\w+ (.*) :(.*)/;
       $xref = $2;
-      &html_postamble; select STDOUT; close TEXT;
-      $html_nav_last = $chapternode;
+      $title = "$2 $1:&ensp;";
+      $title =~ s/-\S+//;
+      $title =~ s/^(.)/\U$1/;
+      html_postamble();
       $chapternode = $nodexrefs{$xref};
-      $html_nav_next = $tstruct_mnext{$chapternode};
-      open(TEXT, '>', File::Spec->catfile($out_path, html_filename($chapternode)));
-      select TEXT;
-      &html_preamble(1);
+      html_preamble($chapternode);
       foreach $i (@$pname) {
-        $ww = &word_html($i);
-        $title .= $ww unless $ww eq "\001";
+	$ww = &word_html($i);
+	$title .= $ww unless $ww eq "\001";
       }
       $h = "<h2 id=\"$xref\">$title</h2>\n";
-      print $h; print FULL $h;
-    } elsif ($ptype eq "appn") {
-      # Appendix heading. Begin a new file.
-      $pflags =~ /appn (.*) :(.*)/;
-      $title = "Appendix $1: ";
-      $xref = $2;
-      &html_postamble; select STDOUT; close TEXT;
-      $html_nav_last = $chapternode;
-      $chapternode = $nodexrefs{$xref};
-      $html_nav_next = $tstruct_mnext{$chapternode};
-      open(TEXT, '>', File::Spec->catfile($out_path, html_filename($chapternode)));
-      select TEXT;
-      &html_preamble(1);
-      foreach $i (@$pname) {
-        $ww = &word_html($i);
-        $title .= $ww unless $ww eq "\001";
-      }
-      print "<h2 id=\"$xref\">$title</h2>\n";
+      print $h;
     } elsif ($ptype eq "head" || $ptype eq "subh") {
       # Heading or subheading.
       $pflags =~ /.... (.*) :(.*)/;
       $hdr = ($ptype eq "subh" ? "h4" : "h3");
-      $title = $1 . " ";
+      $title = $1 . ".&ensp;";
       $xref = $2;
       foreach $i (@$pname) {
         $ww = &word_html($i);
@@ -964,57 +1125,14 @@ sub write_html {
   print "</pre>\n" if ($in_code);
   print "</li>\n</ul>\n" if ($in_list);
   print "</blockquote>\n" if ($in_bquo);
-  &html_postamble; select STDOUT; close TEXT;
+  html_postamble();
 
   print "\n   writing index file...";
-  open TEXT, '>', File::Spec->catfile($out_path, 'nasmdoci.html');
-  select TEXT;
-  &html_preamble(0);
-  print "<h2 class=\"index\">Index</h2>\n";
-  print "<div class=\"index\">\n";
+  html_preamble('index');
+  print "<h2>Index</h2>\n";
   &html_index;
-  print "</div>\n\n</body>\n</html>\n";
-  select STDOUT;
-  close TEXT;
-}
+  html_postamble();
 
-sub html_preamble {
-    print "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>\n";
-    print "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" ";
-    print "\"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n";
-    print "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n";
-    print "<head>\n";
-    print "<meta charset=\"UTF-8\" />\n";
-    print "<title>", $metadata{'title'}, "</title>\n";
-    print "<link href=\"nasmdoc.css\" rel=\"stylesheet\" type=\"text/css\" />\n";
-    print "<link href=\"local.css\" rel=\"stylesheet\" type=\"text/css\" />\n";
-    print "</head>\n";
-    print "<body>\n";
-
-    # Navigation bar
-    print "<ul class=\"navbar\">\n";
-    if (defined($html_nav_last)) {
-	my $lastf = html_filename($html_nav_last);
-	print "<li class=\"first\"><a class=\"prev\" href=\"$lastf\">$html_nav_last</a></li>\n";
-    }
-    if (defined($html_nav_next)) {
-	my $nextf = html_filename($html_nav_next);
-	print "<li><a class=\"next\" href=\"$nextf\">$html_nav_next</a></li>\n";
-    }
-    print "<li><a class=\"toc\" href=\"nasmdoc0.html\">Contents</a></li>\n";
-    print "<li class=\"last\"><a class=\"index\" href=\"nasmdoci.html\">Index</a></li>\n";
-    print "</ul>\n";
-
-    print "<div class=\"title\">\n";
-    print "<h1>", $metadata{'title'}, "</h1>\n";
-    print '<span class="subtitle">', $metadata{'subtitle'}, "</span>\n";
-    print "</div>\n";
-    print "<div class=\"contents\"\n>\n";
-}
-
-sub html_postamble {
-    # Common closing tags
-    print "</div>\n</body>\n</html>\n";
 }
 
 sub html_index {
@@ -1128,7 +1246,7 @@ sub word_html($) {
   } elsif ($wmajt eq "i") {
     return "\001";
   } else {
-    die "panic in word_html: $wtype$w\n";
+    die "$outfile: panic in word_html: $wtype$w\n";
   }
 }
 
