@@ -3,7 +3,7 @@
 ;;; Commentary:
 
 ;; This file contains list of tokens from the NASM x86
-;; assembler, automatically extracted from NASM 3.00.
+;; assembler, automatically extracted from NASM 3.01.
 ;;
 ;; This file is intended to be (require)d from a `nasm-mode'
 ;; major mode definition.
@@ -19,16 +19,17 @@
 (defconst nasm-decorator
   '("{1to16}" "{1to2}" "{1to32}" "{1to4}" "{1to8}" "{rd-sae}" "{rn-sae}"
     "{ru-sae}" "{rz-sae}" "{sae}" "{z}")
-  "NASM 3.00 decorator tokens for `nasm-mode'.")
+  "NASM 3.01 decorator tokens for `nasm-mode'.")
 
 (defconst nasm-directive
   '("absolute" "bits" "common" "cpu" "debug" "default" "dollarhex" "export"
-    "extern" "float" "global" "gprefix" "group" "gsuffix" "import" "library"
-    "limit" "list" "lprefix" "lsuffix" "map" "maxdump" "module"
-    "no_dead_strip" "nodepend" "noseclabels" "options" "org" "osabi" "pragma"
-    "prefix" "required" "safeseh" "sectalign" "section" "segment" "static"
-    "subsections_via_symbols" "suffix" "uppercase" "warning")
-  "NASM 3.00 directive tokens for `nasm-mode'.")
+    "extern" "float" "global" "gpostfix" "gprefix" "group" "gsuffix" "import"
+    "library" "limit" "list" "lpostfix" "lprefix" "lsuffix" "map" "maxdump"
+    "module" "no_dead_strip" "nodepend" "noseclabels" "options" "org" "osabi"
+    "postfix" "pragma" "prefix" "required" "safeseh" "sectalign" "section"
+    "segment" "static" "subsections_via_symbols" "suffix" "uppercase"
+    "warning")
+  "NASM 3.01 directive tokens for `nasm-mode'.")
 
 (defconst nasm-instruction
   '("aaa" "aad" "aadd" "aam" "aand" "aas" "adc" "adcx" "add" "addb" "addd"
@@ -288,33 +289,32 @@
     "vexp2ps" "vexpandpd" "vexpandps" "vextractf128" "vextractf32x4"
     "vextractf32x8" "vextractf64x2" "vextractf64x4" "vextracti128"
     "vextracti32x4" "vextracti32x8" "vextracti64x2" "vextracti64x4"
-    "vextractps" "vfcmaddcph" "vfcmaddcsh" "vfcmulcpch" "vfcmulcph"
-    "vfcmulcsh" "vfixupimmpd" "vfixupimmps" "vfixupimmsd" "vfixupimmss"
-    "vfmadd123pd" "vfmadd123ps" "vfmadd123sd" "vfmadd123ss" "vfmadd132bf16"
-    "vfmadd132pd" "vfmadd132ph" "vfmadd132ps" "vfmadd132sd" "vfmadd132sh"
-    "vfmadd132ss" "vfmadd213bf16" "vfmadd213pd" "vfmadd213ph" "vfmadd213ps"
-    "vfmadd213sd" "vfmadd213sh" "vfmadd213ss" "vfmadd231bf16" "vfmadd231pd"
-    "vfmadd231ph" "vfmadd231ps" "vfmadd231sd" "vfmadd231sh" "vfmadd231ss"
-    "vfmadd312pd" "vfmadd312ps" "vfmadd312sd" "vfmadd312ss" "vfmadd321pd"
-    "vfmadd321ps" "vfmadd321sd" "vfmadd321ss" "vfmaddcph" "vfmaddcsh"
-    "vfmaddpd" "vfmaddps" "vfmaddsd" "vfmaddss" "vfmaddsub123pd"
-    "vfmaddsub123ps" "vfmaddsub132pd" "vfmaddsub132ph" "vfmaddsub132ps"
-    "vfmaddsub213pd" "vfmaddsub213ph" "vfmaddsub213ps" "vfmaddsub231pd"
-    "vfmaddsub231ph" "vfmaddsub231ps" "vfmaddsub312pd" "vfmaddsub312ps"
-    "vfmaddsub321pd" "vfmaddsub321ps" "vfmaddsubpd" "vfmaddsubps"
-    "vfmsub123pd" "vfmsub123ps" "vfmsub123sd" "vfmsub123ss" "vfmsub132bf16"
-    "vfmsub132pd" "vfmsub132ph" "vfmsub132ps" "vfmsub132sd" "vfmsub132sh"
-    "vfmsub132ss" "vfmsub213bf16" "vfmsub213pd" "vfmsub213ph" "vfmsub213ps"
-    "vfmsub213sd" "vfmsub213sh" "vfmsub213ss" "vfmsub231bf16" "vfmsub231pd"
-    "vfmsub231ph" "vfmsub231ps" "vfmsub231sd" "vfmsub231sh" "vfmsub231ss"
-    "vfmsub312pd" "vfmsub312ps" "vfmsub312sd" "vfmsub312ss" "vfmsub321pd"
-    "vfmsub321ps" "vfmsub321sd" "vfmsub321ss" "vfmsubadd123pd"
-    "vfmsubadd123ps" "vfmsubadd132pd" "vfmsubadd132ph" "vfmsubadd132ps"
-    "vfmsubadd213pd" "vfmsubadd213ph" "vfmsubadd213ps" "vfmsubadd231pd"
-    "vfmsubadd231ph" "vfmsubadd231ps" "vfmsubadd312pd" "vfmsubadd312ps"
-    "vfmsubadd321pd" "vfmsubadd321ps" "vfmsubaddpd" "vfmsubaddps" "vfmsubpd"
-    "vfmsubps" "vfmsubsd" "vfmsubss" "vfmulcpch" "vfmulcph" "vfmulcsh"
-    "vfnmadd123pd" "vfnmadd123ps" "vfnmadd123sd" "vfnmadd123ss"
+    "vextractps" "vfcmaddcph" "vfcmaddcsh" "vfcmulcph" "vfcmulcsh"
+    "vfixupimmpd" "vfixupimmps" "vfixupimmsd" "vfixupimmss" "vfmadd123pd"
+    "vfmadd123ps" "vfmadd123sd" "vfmadd123ss" "vfmadd132bf16" "vfmadd132pd"
+    "vfmadd132ph" "vfmadd132ps" "vfmadd132sd" "vfmadd132sh" "vfmadd132ss"
+    "vfmadd213bf16" "vfmadd213pd" "vfmadd213ph" "vfmadd213ps" "vfmadd213sd"
+    "vfmadd213sh" "vfmadd213ss" "vfmadd231bf16" "vfmadd231pd" "vfmadd231ph"
+    "vfmadd231ps" "vfmadd231sd" "vfmadd231sh" "vfmadd231ss" "vfmadd312pd"
+    "vfmadd312ps" "vfmadd312sd" "vfmadd312ss" "vfmadd321pd" "vfmadd321ps"
+    "vfmadd321sd" "vfmadd321ss" "vfmaddcph" "vfmaddcsh" "vfmaddpd" "vfmaddps"
+    "vfmaddsd" "vfmaddss" "vfmaddsub123pd" "vfmaddsub123ps" "vfmaddsub132pd"
+    "vfmaddsub132ph" "vfmaddsub132ps" "vfmaddsub213pd" "vfmaddsub213ph"
+    "vfmaddsub213ps" "vfmaddsub231pd" "vfmaddsub231ph" "vfmaddsub231ps"
+    "vfmaddsub312pd" "vfmaddsub312ps" "vfmaddsub321pd" "vfmaddsub321ps"
+    "vfmaddsubpd" "vfmaddsubps" "vfmsub123pd" "vfmsub123ps" "vfmsub123sd"
+    "vfmsub123ss" "vfmsub132bf16" "vfmsub132pd" "vfmsub132ph" "vfmsub132ps"
+    "vfmsub132sd" "vfmsub132sh" "vfmsub132ss" "vfmsub213bf16" "vfmsub213pd"
+    "vfmsub213ph" "vfmsub213ps" "vfmsub213sd" "vfmsub213sh" "vfmsub213ss"
+    "vfmsub231bf16" "vfmsub231pd" "vfmsub231ph" "vfmsub231ps" "vfmsub231sd"
+    "vfmsub231sh" "vfmsub231ss" "vfmsub312pd" "vfmsub312ps" "vfmsub312sd"
+    "vfmsub312ss" "vfmsub321pd" "vfmsub321ps" "vfmsub321sd" "vfmsub321ss"
+    "vfmsubadd123pd" "vfmsubadd123ps" "vfmsubadd132pd" "vfmsubadd132ph"
+    "vfmsubadd132ps" "vfmsubadd213pd" "vfmsubadd213ph" "vfmsubadd213ps"
+    "vfmsubadd231pd" "vfmsubadd231ph" "vfmsubadd231ps" "vfmsubadd312pd"
+    "vfmsubadd312ps" "vfmsubadd321pd" "vfmsubadd321ps" "vfmsubaddpd"
+    "vfmsubaddps" "vfmsubpd" "vfmsubps" "vfmsubsd" "vfmsubss" "vfmulcph"
+    "vfmulcsh" "vfnmadd123pd" "vfnmadd123ps" "vfnmadd123sd" "vfnmadd123ss"
     "vfnmadd132bf16" "vfnmadd132pd" "vfnmadd132ph" "vfnmadd132ps"
     "vfnmadd132sd" "vfnmadd132sh" "vfnmadd132ss" "vfnmadd213bf16"
     "vfnmadd213pd" "vfnmadd213ph" "vfnmadd213ps" "vfnmadd213sd"
@@ -339,129 +339,124 @@
     "vgatherpf1qps" "vgatherqpd" "vgatherqps" "vgetexpbf16" "vgetexppd"
     "vgetexpph" "vgetexpps" "vgetexpsd" "vgetexpsh" "vgetexpss"
     "vgetmantbf16" "vgetmantpd" "vgetmantph" "vgetmantps" "vgetmantsd"
-    "vgetmantsh" "vgetmantss" "vgetmaxph" "vgetmaxsh" "vgetminph" "vgetminsh"
-    "vgf2p8affineinvqb" "vgf2p8affineqb" "vgf2p8mulb" "vhaddpd" "vhaddps"
-    "vhsubpd" "vhsubps" "vinsertf128" "vinsertf32x4" "vinsertf32x8"
-    "vinsertf64x2" "vinsertf64x4" "vinserti128" "vinserti32x4" "vinserti32x8"
-    "vinserti64x2" "vinserti64x4" "vinsertps" "vlddqu" "vldmxcsr" "vldqqu"
-    "vmaskmovdqu" "vmaskmovpd" "vmaskmovps" "vmaxbf16" "vmaxpd" "vmaxph"
-    "vmaxps" "vmaxsd" "vmaxsh" "vmaxss" "vmcall" "vmclear" "vmfunc" "vmgexit"
-    "vminbf16" "vminmaxbf16" "vminmaxpd" "vminmaxph" "vminmaxps" "vminmaxsd"
-    "vminmaxsh" "vminmaxss" "vminpd" "vminph" "vminps" "vminsd" "vminsh"
-    "vminss" "vmlaunch" "vmload" "vmmcall" "vmovapd" "vmovaps" "vmovd"
-    "vmovddup" "vmovdqa" "vmovdqa32" "vmovdqa64" "vmovdqu" "vmovdqu16"
-    "vmovdqu32" "vmovdqu64" "vmovdqu8" "vmovhlps" "vmovhpd" "vmovhps"
-    "vmovlhps" "vmovlpd" "vmovlps" "vmovmskpd" "vmovmskps" "vmovntdq"
-    "vmovntdqa" "vmovntpd" "vmovntps" "vmovntqq" "vmovq" "vmovqqa" "vmovqqu"
-    "vmovsd" "vmovsh" "vmovshdup" "vmovsldup" "vmovss" "vmovupd" "vmovups"
-    "vmovw" "vmpsadbw" "vmptrld" "vmptrst" "vmread" "vmresume" "vmrun"
-    "vmsave" "vmulbf16" "vmulpd" "vmulph" "vmulps" "vmulsd" "vmulsh" "vmulss"
-    "vmwrite" "vmxoff" "vmxon" "vorpd" "vorps" "vp2intersectd"
-    "vp2intersectq" "vpabsb" "vpabsd" "vpabsq" "vpabsw" "vpackssdw"
-    "vpacksswb" "vpackusdw" "vpackuswb" "vpaddb" "vpaddd" "vpaddq" "vpaddsb"
-    "vpaddsw" "vpaddusb" "vpaddusw" "vpaddw" "vpalignr" "vpand" "vpandd"
-    "vpandn" "vpandnd" "vpandnq" "vpandq" "vpavgb" "vpavgw" "vpblendd"
-    "vpblendmb" "vpblendmd" "vpblendmq" "vpblendmw" "vpblendvb" "vpblendw"
-    "vpbroadcastb" "vpbroadcastd" "vpbroadcastmb2q" "vpbroadcastmw2d"
-    "vpbroadcastq" "vpbroadcastw" "vpclmulhqhqdq" "vpclmulhqlqdq"
-    "vpclmullqhqdq" "vpclmullqlqdq" "vpclmulqdq" "vpcmov" "vpcmpb" "vpcmpd"
-    "vpcmpeqb" "vpcmpeqd" "vpcmpeqq" "vpcmpequb" "vpcmpequd" "vpcmpequq"
-    "vpcmpequw" "vpcmpeqw" "vpcmpestri" "vpcmpestrm" "vpcmpgeb" "vpcmpged"
-    "vpcmpgeq" "vpcmpgeub" "vpcmpgeud" "vpcmpgeuq" "vpcmpgeuw" "vpcmpgew"
-    "vpcmpgtb" "vpcmpgtd" "vpcmpgtq" "vpcmpgtub" "vpcmpgtud" "vpcmpgtuq"
-    "vpcmpgtuw" "vpcmpgtw" "vpcmpistri" "vpcmpistrm" "vpcmpleb" "vpcmpled"
-    "vpcmpleq" "vpcmpleub" "vpcmpleud" "vpcmpleuq" "vpcmpleuw" "vpcmplew"
-    "vpcmpltb" "vpcmpltd" "vpcmpltq" "vpcmpltub" "vpcmpltud" "vpcmpltuq"
-    "vpcmpltuw" "vpcmpltw" "vpcmpneqb" "vpcmpneqd" "vpcmpneqq" "vpcmpnequb"
-    "vpcmpnequd" "vpcmpnequq" "vpcmpnequw" "vpcmpneqw" "vpcmpngtb"
-    "vpcmpngtd" "vpcmpngtq" "vpcmpngtub" "vpcmpngtud" "vpcmpngtuq"
-    "vpcmpngtuw" "vpcmpngtw" "vpcmpnleb" "vpcmpnled" "vpcmpnleq" "vpcmpnleub"
-    "vpcmpnleud" "vpcmpnleuq" "vpcmpnleuw" "vpcmpnlew" "vpcmpnltb"
-    "vpcmpnltd" "vpcmpnltq" "vpcmpnltub" "vpcmpnltud" "vpcmpnltuq"
-    "vpcmpnltuw" "vpcmpnltw" "vpcmpq" "vpcmpub" "vpcmpud" "vpcmpuq" "vpcmpuw"
-    "vpcmpw" "vpcomb" "vpcomd" "vpcompressb" "vpcompressd" "vpcompressq"
-    "vpcompressw" "vpcomq" "vpcomub" "vpcomud" "vpcomuq" "vpcomuw" "vpcomw"
-    "vpconflictd" "vpconflictq" "vpdpbssd" "vpdpbssds" "vpdpbsud" "vpdpbsuds"
-    "vpdpbusd" "vpdpbusds" "vpdpbuud" "vpdpbuuds" "vpdpwssd" "vpdpwssds"
-    "vpdpwsud" "vpdpwsuds" "vpdpwusd" "vpdpwusds" "vpdpwuud" "vpdpwuuds"
-    "vperm2f128" "vperm2i128" "vpermb" "vpermd" "vpermi2b" "vpermi2d"
-    "vpermi2pd" "vpermi2ps" "vpermi2q" "vpermi2w" "vpermilpd" "vpermilps"
-    "vpermpd" "vpermps" "vpermq" "vpermt2b" "vpermt2d" "vpermt2pd"
-    "vpermt2ps" "vpermt2q" "vpermt2w" "vpermw" "vpexpandb" "vpexpandd"
-    "vpexpandq" "vpexpandw" "vpextrb" "vpextrd" "vpextrq" "vpextrw"
-    "vpgatherdd" "vpgatherdq" "vpgatherqd" "vpgatherqq" "vphaddbd" "vphaddbq"
-    "vphaddbw" "vphaddd" "vphadddq" "vphaddsw" "vphaddubd" "vphaddubq"
-    "vphaddubw" "vphaddudq" "vphadduwd" "vphadduwq" "vphaddw" "vphaddwd"
-    "vphaddwq" "vphminposuw" "vphsubbw" "vphsubd" "vphsubdq" "vphsubsw"
-    "vphsubw" "vphsubwd" "vpinsrb" "vpinsrd" "vpinsrq" "vpinsrw" "vplzcntd"
-    "vplzcntq" "vpmacsdd" "vpmacsdqh" "vpmacsdql" "vpmacssdd" "vpmacssdqh"
-    "vpmacssdql" "vpmacsswd" "vpmacssww" "vpmacswd" "vpmacsww" "vpmadcsswd"
-    "vpmadcswd" "vpmadd132ph" "vpmadd132sh" "vpmadd213ph" "vpmadd213sh"
-    "vpmadd231ph" "vpmadd231sh" "vpmadd52huq" "vpmadd52luq" "vpmaddubsw"
-    "vpmaddwd" "vpmaskmovd" "vpmaskmovq" "vpmaxsb" "vpmaxsd" "vpmaxsq"
-    "vpmaxsw" "vpmaxub" "vpmaxud" "vpmaxuq" "vpmaxuw" "vpminsb" "vpminsd"
-    "vpminsq" "vpminsw" "vpminub" "vpminud" "vpminuq" "vpminuw" "vpmovb2m"
-    "vpmovd2m" "vpmovdb" "vpmovdw" "vpmovm2b" "vpmovm2d" "vpmovm2q"
-    "vpmovm2w" "vpmovmskb" "vpmovq2m" "vpmovqb" "vpmovqd" "vpmovqw"
-    "vpmovsdb" "vpmovsdw" "vpmovsqb" "vpmovsqd" "vpmovsqw" "vpmovswb"
-    "vpmovsxbd" "vpmovsxbq" "vpmovsxbw" "vpmovsxdq" "vpmovsxwd" "vpmovsxwq"
-    "vpmovusdb" "vpmovusdw" "vpmovusqb" "vpmovusqd" "vpmovusqw" "vpmovuswb"
-    "vpmovw2m" "vpmovwb" "vpmovzxbd" "vpmovzxbq" "vpmovzxbw" "vpmovzxdq"
-    "vpmovzxwd" "vpmovzxwq" "vpmsub132ph" "vpmsub132sh" "vpmsub213ph"
-    "vpmsub213sh" "vpmsub231ph" "vpmsub231sh" "vpmuldq" "vpmulhrsw"
-    "vpmulhuw" "vpmulhw" "vpmulld" "vpmullq" "vpmullw" "vpmultishiftqb"
-    "vpmuludq" "vpnmadd132sh" "vpnmadd213sh" "vpnmadd231sh" "vpnmsub132sh"
-    "vpnmsub213sh" "vpnmsub231sh" "vpopcntb" "vpopcntd" "vpopcntq" "vpopcntw"
-    "vpor" "vpord" "vporq" "vpperm" "vprold" "vprolq" "vprolvd" "vprolvq"
-    "vprord" "vprorq" "vprorvd" "vprorvq" "vprotb" "vprotd" "vprotq" "vprotw"
-    "vpsadbw" "vpscatterdd" "vpscatterdq" "vpscatterqd" "vpscatterqq"
-    "vpshab" "vpshad" "vpshaq" "vpshaw" "vpshlb" "vpshld" "vpshldd" "vpshldq"
-    "vpshldvd" "vpshldvq" "vpshldvw" "vpshldw" "vpshlq" "vpshlw" "vpshrdd"
-    "vpshrdq" "vpshrdvd" "vpshrdvq" "vpshrdvw" "vpshrdw" "vpshufb"
-    "vpshufbitqmb" "vpshufd" "vpshufhw" "vpshuflw" "vpsignb" "vpsignd"
-    "vpsignw" "vpslld" "vpslldq" "vpsllq" "vpsllvd" "vpsllvq" "vpsllvw"
-    "vpsllw" "vpsrad" "vpsraq" "vpsravd" "vpsravq" "vpsravw" "vpsraw"
-    "vpsrld" "vpsrldq" "vpsrlq" "vpsrlvd" "vpsrlvq" "vpsrlvw" "vpsrlw"
-    "vpsubb" "vpsubd" "vpsubq" "vpsubsb" "vpsubsw" "vpsubusb" "vpsubusw"
-    "vpsubw" "vpternlogd" "vpternlogq" "vptest" "vptestmb" "vptestmd"
-    "vptestmq" "vptestmw" "vptestnmb" "vptestnmd" "vptestnmq" "vptestnmw"
-    "vpunpckhbw" "vpunpckhdq" "vpunpckhqdq" "vpunpckhwd" "vpunpcklbw"
-    "vpunpckldq" "vpunpcklqdq" "vpunpcklwd" "vpxor" "vpxord" "vpxorq"
-    "vrangepd" "vrangeps" "vrangesd" "vrangess" "vrcp14pd" "vrcp14ps"
-    "vrcp14sd" "vrcp14ss" "vrcp28pd" "vrcp28ps" "vrcp28sd" "vrcp28ss"
-    "vrcpbf16" "vrcpph" "vrcpps" "vrcpsh" "vrcpss" "vreducebf16" "vreducepd"
-    "vreduceph" "vreduceps" "vreducesd" "vreducesh" "vreducess"
-    "vrndscalebf16" "vrndscalepd" "vrndscaleph" "vrndscaleps" "vrndscalesd"
-    "vrndscalesh" "vrndscaless" "vroundpd" "vroundps" "vroundsd" "vroundss"
-    "vrsqrt14pd" "vrsqrt14ps" "vrsqrt14sd" "vrsqrt14ss" "vrsqrt28pd"
-    "vrsqrt28ps" "vrsqrt28sd" "vrsqrt28ss" "vrsqrtbf16" "vrsqrtph" "vrsqrtps"
-    "vrsqrtsh" "vrsqrtss" "vscalefbf16" "vscalefpd" "vscalefph" "vscalefps"
-    "vscalefsd" "vscalefsh" "vscalefss" "vscatterdpd" "vscatterdps"
-    "vscatterpf0dpd" "vscatterpf0dps" "vscatterpf0qpd" "vscatterpf0qps"
-    "vscatterpf1dpd" "vscatterpf1dps" "vscatterpf1qpd" "vscatterpf1qps"
-    "vscatterqpd" "vscatterqps" "vsha512msg1" "vsha512msg2" "vsha512rnds2"
-    "vshuff32x4" "vshuff64x2" "vshufi32x4" "vshufi64x2" "vshufpd" "vshufps"
-    "vsm3msg1" "vsm3msg2" "vsm3rnds2" "vsm4key4" "vsm4rnds4" "vsqrtbf16"
-    "vsqrtpd" "vsqrtph" "vsqrtps" "vsqrtsd" "vsqrtsh" "vsqrtss" "vstmxcsr"
-    "vsubbf16" "vsubpd" "vsubph" "vsubps" "vsubsd" "vsubsh" "vsubss"
-    "vtestpd" "vtestps" "vucomisd" "vucomish" "vucomiss" "vucomxsd"
-    "vucomxsh" "vucomxss" "vunpckhpd" "vunpckhps" "vunpcklpd" "vunpcklps"
-    "vxorpd" "vxorps" "vzeroall" "vzeroupper" "wbinvd" "wbnoinvd" "wrfsbase"
-    "wrgsbase" "wrmsr" "wrmsrlist" "wrmsrns" "wrpkru" "wrshr" "wrssd" "wrssq"
-    "wrussd" "wrussq" "xabort" "xadd" "xbegin" "xbts" "xchg" "xcryptcbc"
-    "xcryptcfb" "xcryptctr" "xcryptecb" "xcryptofb" "xend" "xgetbv" "xlat"
-    "xlatb" "xnor" "xnorb" "xnord" "xnorq" "xnorw" "xor" "xorb" "xord"
-    "xorpd" "xorps" "xorq" "xorw" "xresldtrk" "xrstor" "xrstor64" "xrstors"
-    "xrstors64" "xsave" "xsave64" "xsavec" "xsavec64" "xsaveopt" "xsaveopt64"
-    "xsaves" "xsaves64" "xsetbv" "xsha1" "xsha256" "xstore" "xsusldtrk"
-    "xtest")
-  "NASM 3.00 instruction tokens for `nasm-mode'.")
+    "vgetmantsh" "vgetmantss" "vgf2p8affineinvqb" "vgf2p8affineqb"
+    "vgf2p8mulb" "vhaddpd" "vhaddps" "vhsubpd" "vhsubps" "vinsertf128"
+    "vinsertf32x4" "vinsertf32x8" "vinsertf64x2" "vinsertf64x4" "vinserti128"
+    "vinserti32x4" "vinserti32x8" "vinserti64x2" "vinserti64x4" "vinsertps"
+    "vlddqu" "vldmxcsr" "vldqqu" "vmaskmovdqu" "vmaskmovpd" "vmaskmovps"
+    "vmaxbf16" "vmaxpd" "vmaxph" "vmaxps" "vmaxsd" "vmaxsh" "vmaxss" "vmcall"
+    "vmclear" "vmfunc" "vmgexit" "vminbf16" "vminmaxbf16" "vminmaxpd"
+    "vminmaxph" "vminmaxps" "vminmaxsd" "vminmaxsh" "vminmaxss" "vminpd"
+    "vminph" "vminps" "vminsd" "vminsh" "vminss" "vmlaunch" "vmload"
+    "vmmcall" "vmovapd" "vmovaps" "vmovd" "vmovddup" "vmovdqa" "vmovdqa32"
+    "vmovdqa64" "vmovdqu" "vmovdqu16" "vmovdqu32" "vmovdqu64" "vmovdqu8"
+    "vmovhlps" "vmovhpd" "vmovhps" "vmovlhps" "vmovlpd" "vmovlps" "vmovmskpd"
+    "vmovmskps" "vmovntdq" "vmovntdqa" "vmovntpd" "vmovntps" "vmovntqq"
+    "vmovq" "vmovqqa" "vmovqqu" "vmovsd" "vmovsh" "vmovshdup" "vmovsldup"
+    "vmovss" "vmovupd" "vmovups" "vmovw" "vmpsadbw" "vmptrld" "vmptrst"
+    "vmread" "vmresume" "vmrun" "vmsave" "vmulbf16" "vmulpd" "vmulph"
+    "vmulps" "vmulsd" "vmulsh" "vmulss" "vmwrite" "vmxoff" "vmxon" "vorpd"
+    "vorps" "vp2intersectd" "vp2intersectq" "vpabsb" "vpabsd" "vpabsq"
+    "vpabsw" "vpackssdw" "vpacksswb" "vpackusdw" "vpackuswb" "vpaddb"
+    "vpaddd" "vpaddq" "vpaddsb" "vpaddsw" "vpaddusb" "vpaddusw" "vpaddw"
+    "vpalignr" "vpand" "vpandd" "vpandn" "vpandnd" "vpandnq" "vpandq"
+    "vpavgb" "vpavgw" "vpblendd" "vpblendmb" "vpblendmd" "vpblendmq"
+    "vpblendmw" "vpblendvb" "vpblendw" "vpbroadcastb" "vpbroadcastd"
+    "vpbroadcastmb2q" "vpbroadcastmw2d" "vpbroadcastq" "vpbroadcastw"
+    "vpclmulhqhqdq" "vpclmulhqlqdq" "vpclmullqhqdq" "vpclmullqlqdq"
+    "vpclmulqdq" "vpcmov" "vpcmpb" "vpcmpd" "vpcmpeqb" "vpcmpeqd" "vpcmpeqq"
+    "vpcmpequb" "vpcmpequd" "vpcmpequq" "vpcmpequw" "vpcmpeqw" "vpcmpestri"
+    "vpcmpestrm" "vpcmpgeb" "vpcmpged" "vpcmpgeq" "vpcmpgeub" "vpcmpgeud"
+    "vpcmpgeuq" "vpcmpgeuw" "vpcmpgew" "vpcmpgtb" "vpcmpgtd" "vpcmpgtq"
+    "vpcmpgtub" "vpcmpgtud" "vpcmpgtuq" "vpcmpgtuw" "vpcmpgtw" "vpcmpistri"
+    "vpcmpistrm" "vpcmpleb" "vpcmpled" "vpcmpleq" "vpcmpleub" "vpcmpleud"
+    "vpcmpleuq" "vpcmpleuw" "vpcmplew" "vpcmpltb" "vpcmpltd" "vpcmpltq"
+    "vpcmpltub" "vpcmpltud" "vpcmpltuq" "vpcmpltuw" "vpcmpltw" "vpcmpneqb"
+    "vpcmpneqd" "vpcmpneqq" "vpcmpnequb" "vpcmpnequd" "vpcmpnequq"
+    "vpcmpnequw" "vpcmpneqw" "vpcmpngtb" "vpcmpngtd" "vpcmpngtq" "vpcmpngtub"
+    "vpcmpngtud" "vpcmpngtuq" "vpcmpngtuw" "vpcmpngtw" "vpcmpnleb"
+    "vpcmpnled" "vpcmpnleq" "vpcmpnleub" "vpcmpnleud" "vpcmpnleuq"
+    "vpcmpnleuw" "vpcmpnlew" "vpcmpnltb" "vpcmpnltd" "vpcmpnltq" "vpcmpnltub"
+    "vpcmpnltud" "vpcmpnltuq" "vpcmpnltuw" "vpcmpnltw" "vpcmpq" "vpcmpub"
+    "vpcmpud" "vpcmpuq" "vpcmpuw" "vpcmpw" "vpcomb" "vpcomd" "vpcompressb"
+    "vpcompressd" "vpcompressq" "vpcompressw" "vpcomq" "vpcomub" "vpcomud"
+    "vpcomuq" "vpcomuw" "vpcomw" "vpconflictd" "vpconflictq" "vpdpbssd"
+    "vpdpbssds" "vpdpbsud" "vpdpbsuds" "vpdpbusd" "vpdpbusds" "vpdpbuud"
+    "vpdpbuuds" "vpdpwssd" "vpdpwssds" "vpdpwsud" "vpdpwsuds" "vpdpwusd"
+    "vpdpwusds" "vpdpwuud" "vpdpwuuds" "vperm2f128" "vperm2i128" "vpermb"
+    "vpermd" "vpermi2b" "vpermi2d" "vpermi2pd" "vpermi2ps" "vpermi2q"
+    "vpermi2w" "vpermilpd" "vpermilps" "vpermpd" "vpermps" "vpermq"
+    "vpermt2b" "vpermt2d" "vpermt2pd" "vpermt2ps" "vpermt2q" "vpermt2w"
+    "vpermw" "vpexpandb" "vpexpandd" "vpexpandq" "vpexpandw" "vpextrb"
+    "vpextrd" "vpextrq" "vpextrw" "vpgatherdd" "vpgatherdq" "vpgatherqd"
+    "vpgatherqq" "vphaddbd" "vphaddbq" "vphaddbw" "vphaddd" "vphadddq"
+    "vphaddsw" "vphaddubd" "vphaddubq" "vphaddubw" "vphaddudq" "vphadduwd"
+    "vphadduwq" "vphaddw" "vphaddwd" "vphaddwq" "vphminposuw" "vphsubbw"
+    "vphsubd" "vphsubdq" "vphsubsw" "vphsubw" "vphsubwd" "vpinsrb" "vpinsrd"
+    "vpinsrq" "vpinsrw" "vplzcntd" "vplzcntq" "vpmacsdd" "vpmacsdqh"
+    "vpmacsdql" "vpmacssdd" "vpmacssdqh" "vpmacssdql" "vpmacsswd" "vpmacssww"
+    "vpmacswd" "vpmacsww" "vpmadcsswd" "vpmadcswd" "vpmadd52huq"
+    "vpmadd52luq" "vpmaddubsw" "vpmaddwd" "vpmaskmovd" "vpmaskmovq" "vpmaxsb"
+    "vpmaxsd" "vpmaxsq" "vpmaxsw" "vpmaxub" "vpmaxud" "vpmaxuq" "vpmaxuw"
+    "vpminsb" "vpminsd" "vpminsq" "vpminsw" "vpminub" "vpminud" "vpminuq"
+    "vpminuw" "vpmovb2m" "vpmovd2m" "vpmovdb" "vpmovdw" "vpmovm2b" "vpmovm2d"
+    "vpmovm2q" "vpmovm2w" "vpmovmskb" "vpmovq2m" "vpmovqb" "vpmovqd"
+    "vpmovqw" "vpmovsdb" "vpmovsdw" "vpmovsqb" "vpmovsqd" "vpmovsqw"
+    "vpmovswb" "vpmovsxbd" "vpmovsxbq" "vpmovsxbw" "vpmovsxdq" "vpmovsxwd"
+    "vpmovsxwq" "vpmovusdb" "vpmovusdw" "vpmovusqb" "vpmovusqd" "vpmovusqw"
+    "vpmovuswb" "vpmovw2m" "vpmovwb" "vpmovzxbd" "vpmovzxbq" "vpmovzxbw"
+    "vpmovzxdq" "vpmovzxwd" "vpmovzxwq" "vpmuldq" "vpmulhrsw" "vpmulhuw"
+    "vpmulhw" "vpmulld" "vpmullq" "vpmullw" "vpmultishiftqb" "vpmuludq"
+    "vpopcntb" "vpopcntd" "vpopcntq" "vpopcntw" "vpor" "vpord" "vporq"
+    "vpperm" "vprold" "vprolq" "vprolvd" "vprolvq" "vprord" "vprorq"
+    "vprorvd" "vprorvq" "vprotb" "vprotd" "vprotq" "vprotw" "vpsadbw"
+    "vpscatterdd" "vpscatterdq" "vpscatterqd" "vpscatterqq" "vpshab" "vpshad"
+    "vpshaq" "vpshaw" "vpshlb" "vpshld" "vpshldd" "vpshldq" "vpshldvd"
+    "vpshldvq" "vpshldvw" "vpshldw" "vpshlq" "vpshlw" "vpshrdd" "vpshrdq"
+    "vpshrdvd" "vpshrdvq" "vpshrdvw" "vpshrdw" "vpshufb" "vpshufbitqmb"
+    "vpshufd" "vpshufhw" "vpshuflw" "vpsignb" "vpsignd" "vpsignw" "vpslld"
+    "vpslldq" "vpsllq" "vpsllvd" "vpsllvq" "vpsllvw" "vpsllw" "vpsrad"
+    "vpsraq" "vpsravd" "vpsravq" "vpsravw" "vpsraw" "vpsrld" "vpsrldq"
+    "vpsrlq" "vpsrlvd" "vpsrlvq" "vpsrlvw" "vpsrlw" "vpsubb" "vpsubd"
+    "vpsubq" "vpsubsb" "vpsubsw" "vpsubusb" "vpsubusw" "vpsubw" "vpternlogd"
+    "vpternlogq" "vptest" "vptestmb" "vptestmd" "vptestmq" "vptestmw"
+    "vptestnmb" "vptestnmd" "vptestnmq" "vptestnmw" "vpunpckhbw" "vpunpckhdq"
+    "vpunpckhqdq" "vpunpckhwd" "vpunpcklbw" "vpunpckldq" "vpunpcklqdq"
+    "vpunpcklwd" "vpxor" "vpxord" "vpxorq" "vrangepd" "vrangeps" "vrangesd"
+    "vrangess" "vrcp14pd" "vrcp14ps" "vrcp14sd" "vrcp14ss" "vrcp28pd"
+    "vrcp28ps" "vrcp28sd" "vrcp28ss" "vrcpbf16" "vrcpph" "vrcpps" "vrcpsh"
+    "vrcpss" "vreducebf16" "vreducepd" "vreduceph" "vreduceps" "vreducesd"
+    "vreducesh" "vreducess" "vrndscalebf16" "vrndscalepd" "vrndscaleph"
+    "vrndscaleps" "vrndscalesd" "vrndscalesh" "vrndscaless" "vroundpd"
+    "vroundps" "vroundsd" "vroundss" "vrsqrt14pd" "vrsqrt14ps" "vrsqrt14sd"
+    "vrsqrt14ss" "vrsqrt28pd" "vrsqrt28ps" "vrsqrt28sd" "vrsqrt28ss"
+    "vrsqrtbf16" "vrsqrtph" "vrsqrtps" "vrsqrtsh" "vrsqrtss" "vscalefbf16"
+    "vscalefpd" "vscalefph" "vscalefps" "vscalefsd" "vscalefsh" "vscalefss"
+    "vscatterdpd" "vscatterdps" "vscatterpf0dpd" "vscatterpf0dps"
+    "vscatterpf0qpd" "vscatterpf0qps" "vscatterpf1dpd" "vscatterpf1dps"
+    "vscatterpf1qpd" "vscatterpf1qps" "vscatterqpd" "vscatterqps"
+    "vsha512msg1" "vsha512msg2" "vsha512rnds2" "vshuff32x4" "vshuff64x2"
+    "vshufi32x4" "vshufi64x2" "vshufpd" "vshufps" "vsm3msg1" "vsm3msg2"
+    "vsm3rnds2" "vsm4key4" "vsm4rnds4" "vsqrtbf16" "vsqrtpd" "vsqrtph"
+    "vsqrtps" "vsqrtsd" "vsqrtsh" "vsqrtss" "vstmxcsr" "vsubbf16" "vsubpd"
+    "vsubph" "vsubps" "vsubsd" "vsubsh" "vsubss" "vtestpd" "vtestps"
+    "vucomisd" "vucomish" "vucomiss" "vucomxsd" "vucomxsh" "vucomxss"
+    "vunpckhpd" "vunpckhps" "vunpcklpd" "vunpcklps" "vxorpd" "vxorps"
+    "vzeroall" "vzeroupper" "wbinvd" "wbnoinvd" "wrfsbase" "wrgsbase" "wrmsr"
+    "wrmsrlist" "wrmsrns" "wrpkru" "wrshr" "wrssd" "wrssq" "wrussd" "wrussq"
+    "xabort" "xadd" "xbegin" "xbts" "xchg" "xcryptcbc" "xcryptcfb"
+    "xcryptctr" "xcryptecb" "xcryptofb" "xend" "xgetbv" "xlat" "xlatb" "xnor"
+    "xnorb" "xnord" "xnorq" "xnorw" "xor" "xorb" "xord" "xorpd" "xorps"
+    "xorq" "xorw" "xresldtrk" "xrstor" "xrstor64" "xrstors" "xrstors64"
+    "xsave" "xsave64" "xsavec" "xsavec64" "xsaveopt" "xsaveopt64" "xsaves"
+    "xsaves64" "xsetbv" "xsha1" "xsha256" "xstore" "xsusldtrk" "xtest")
+  "NASM 3.01 instruction tokens for `nasm-mode'.")
 
 (defconst nasm-mmacro
   '("__?NASM_CDecl?__" "align" "alignb" "alignmode" "at" "bf16"
     "build_version" "end" "endp" "ends" "endstruc" "iend" "istruc" "proc"
     "struc" "use16" "use32" "use64" "useabs" "usebnd" "usenobnd" "userel")
-  "NASM 3.00 mmacro tokens for `nasm-mode'.")
+  "NASM 3.01 mmacro tokens for `nasm-mode'.")
 
 (defconst nasm-operator
   '("!=" "%%" "&&" "//" "<<" "<<<" "<=" "<=>" "<>" "==" ">=" ">>" ">>>" "?"
@@ -475,7 +470,7 @@
     "float80m" "ilog2c" "ilog2e" "ilog2f" "ilog2w" "infinity" "masm_flat"
     "masm_ptr" "nan" "qnan" "snan" "utf16" "utf16be" "utf16le" "utf32"
     "utf32be" "utf32le" "||")
-  "NASM 3.00 operator tokens for `nasm-mode'.")
+  "NASM 3.01 operator tokens for `nasm-mode'.")
 
 (defconst nasm-pp-directive
   '("%aliases" "%arg" "%assign" "%clear" "%defalias" "%define" "%defstr"
@@ -499,13 +494,13 @@
     "%pathsearch" "%pop" "%pragma" "%push" "%rep" "%repl" "%require"
     "%rmacro" "%rotate" "%stacksize" "%strcat" "%strlen" "%substr" "%undef"
     "%undefalias" "%unimacro" "%unmacro" "%use" "%warning" "%xdefine")
-  "NASM 3.00 pp-directive tokens for `nasm-mode'.")
+  "NASM 3.01 pp-directive tokens for `nasm-mode'.")
 
 (defconst nasm-prefix
   '("a16" "a32" "a64" "asp" "bnd" "lock" "nobnd" "o16" "o32" "o64" "osp"
     "rep" "repe" "repne" "repnz" "repz" "wait" "xacquire" "xrelease" "{evex}"
     "{nf}" "{pn}" "{pt}" "{rex2}" "{rex}" "{vex2}" "{vex3}" "{vex}" "{zu}")
-  "NASM 3.00 prefix tokens for `nasm-mode'.")
+  "NASM 3.01 prefix tokens for `nasm-mode'.")
 
 (defconst nasm-register
   '("ah" "al" "ax" "bh" "bl" "bnd0" "bnd1" "bnd2" "bnd3" "bp" "bpl" "bx" "ch"
@@ -541,11 +536,11 @@
     "zmm17" "zmm18" "zmm19" "zmm2" "zmm20" "zmm21" "zmm22" "zmm23" "zmm24"
     "zmm25" "zmm26" "zmm27" "zmm28" "zmm29" "zmm3" "zmm30" "zmm31" "zmm4"
     "zmm5" "zmm6" "zmm7" "zmm8" "zmm9")
-  "NASM 3.00 register tokens for `nasm-mode'.")
+  "NASM 3.01 register tokens for `nasm-mode'.")
 
 (defconst nasm-size
   '("byte" "dword" "long" "oword" "qword" "tword" "word" "yword" "zword")
-  "NASM 3.00 size tokens for `nasm-mode'.")
+  "NASM 3.01 size tokens for `nasm-mode'.")
 
 (defconst nasm-smacro
   '("%abs" "%b2hs" "%chr" "%cond" "%count" "%eval" "%find" "%findi" "%hex"
@@ -555,15 +550,15 @@
     "%isnempty" "%isnenv" "%isnfile" "%isnid" "%isnidn" "%isnidni"
     "%isnmacro" "%isnnum" "%isnstr" "%isntoken" "%isnum" "%isnusable"
     "%isnusing" "%isstr" "%istoken" "%isusable" "%isusing" "%map" "%num"
-    "%ord" "%realpath" "%sel" "%str" "%tok" "Inf" "NaN" "QNaN" "SNaN"
-    "__?BITS?__" "__?DATE?__" "__?DATE_NUM?__" "__?DEFAULT?__" "__?FILE?__"
-    "__?FLOAT?__" "__?FLOAT_DAZ?__" "__?FLOAT_ROUND?__" "__?LINE?__"
-    "__?NASM_HAS_IFDIRECTIVE?__" "__?NASM_MAJOR?__" "__?NASM_MINOR?__"
-    "__?NASM_PATCHLEVEL?__" "__?NASM_SNAPSHOT?__" "__?NASM_SUBMINOR?__"
-    "__?NASM_VER?__" "__?NASM_VERSION_ID?__" "__?OUTPUT_FORMAT?__"
-    "__?PASS?__" "__?POSIX_TIME?__" "__?PTR?__" "__?SECT?__"
-    "__?SECTALIGN_ALIGN_UPDATES_SECTION?__" "__?TIME?__" "__?TIME_NUM?__"
-    "__?UTC_DATE?__" "__?UTC_DATE_NUM?__" "__?UTC_TIME?__"
+    "%ord" "%realpath" "%sel" "%selbits" "%str" "%tok" "Inf" "NaN" "QNaN"
+    "SNaN" "__?BITS?__" "__?DATE?__" "__?DATE_NUM?__" "__?DEFAULT?__"
+    "__?FILE?__" "__?FLOAT?__" "__?FLOAT_DAZ?__" "__?FLOAT_ROUND?__"
+    "__?LINE?__" "__?NASM_HAS_IFDIRECTIVE?__" "__?NASM_MAJOR?__"
+    "__?NASM_MINOR?__" "__?NASM_PATCHLEVEL?__" "__?NASM_SNAPSHOT?__"
+    "__?NASM_SUBMINOR?__" "__?NASM_VER?__" "__?NASM_VERSION_ID?__"
+    "__?OUTPUT_FORMAT?__" "__?PASS?__" "__?POSIX_TIME?__" "__?PTR?__"
+    "__?SECT?__" "__?SECTALIGN_ALIGN_UPDATES_SECTION?__" "__?TIME?__"
+    "__?TIME_NUM?__" "__?UTC_DATE?__" "__?UTC_DATE_NUM?__" "__?UTC_TIME?__"
     "__?UTC_TIME_NUM?__" "__ALIGNMODE__" "__BITS__" "__DATE_NUM__" "__DATE__"
     "__DEBUG_FORMAT__" "__FILE__" "__FLOAT_DAZ__" "__FLOAT_ROUND__"
     "__FLOAT__" "__LINE__" "__NASM_MAJOR__" "__NASM_MINOR__"
@@ -583,28 +578,28 @@
     "r30l" "r31l" "r3b" "r3d" "r3h" "r3l" "r3w" "r4" "r4b" "r4d" "r4l" "r4w"
     "r5" "r5b" "r5d" "r5l" "r5w" "r6" "r6b" "r6d" "r6l" "r6w" "r7" "r7b"
     "r7d" "r7l" "r7w" "r8l" "r9l" "tbyte")
-  "NASM 3.00 smacro tokens for `nasm-mode'.")
+  "NASM 3.01 smacro tokens for `nasm-mode'.")
 
 (defconst nasm-special
   '("abs" "dup" "far" "near" "nosplit" "ptr" "rel" "seg" "short" "strict"
     "times" "to" "wrt")
-  "NASM 3.00 special tokens for `nasm-mode'.")
+  "NASM 3.01 special tokens for `nasm-mode'.")
 
 (defconst nasm-special-constant
   '("{dfv=}")
-  "NASM 3.00 special-constant tokens for `nasm-mode'.")
+  "NASM 3.01 special-constant tokens for `nasm-mode'.")
 
 (defconst nasm-token-lists
   '('nasm-decorator 'nasm-directive 'nasm-instruction 'nasm-mmacro
     'nasm-operator 'nasm-pp-directive 'nasm-prefix 'nasm-register 'nasm-size
     'nasm-smacro 'nasm-special 'nasm-special-constant)
-  "List of all NASM 3.00 token type lists.")
+  "List of all NASM 3.01 token type lists.")
 
-(defconst nasm-token-version "3.00"
+(defconst nasm-token-version "3.01"
   "Version of NASM from which tokens were extracted,
 as a human-readable string.")
 
-(defconst nasm-token-version-id #x03000000
+(defconst nasm-token-version-id #x03010000
   "Version of NASM from which tokens were extracted,
 as numeric identifier, for comparisons. Equivalent to the
 __?NASM_VERSION_ID?__ NASM macro value.")
