@@ -15,7 +15,7 @@
  * OF_ONLY                -- only include specified object formats
  * OF_name                -- ensure that output format 'name' is included
  * OF_NO_name             -- remove output format 'name'
- * OF_DOS                 -- ensure that 'obj', 'bin', 'win32' & 'win64' are included.
+ * OF_DOS                 -- ensure that 'obj', 'obj2', 'bin', 'win32' & 'win64' are included.
  * OF_UNIX                -- ensure that 'aout', 'aoutb', 'coff', 'elf32' & 'elf64' are in.
  * OF_OTHERS              -- ensure that 'bin', 'as86', 'rdf' 'macho32' & 'macho64' are in.
  * OF_ALL                 -- ensure that all formats are included.
@@ -56,7 +56,7 @@
 
 /* ====configurable info begins here==== */
 /* formats configurable:
- * bin,obj,elf32,elf64,aout,aoutb,coff,win32,as86,rdf2,macho32,macho64 */
+ * bin,obj,obj2,elf32,elf64,aout,aoutb,coff,win32,as86,rdf2,macho32,macho64 */
 
 /* process options... */
 
@@ -73,6 +73,9 @@
 #ifndef OF_OBJ
 #define OF_OBJ
 #endif
+#ifndef OF_OBJ2
+#define OF_OBJ2
+#endif
 #ifndef OF_ELF32
 #define OF_ELF32
 #endif
@@ -119,6 +122,9 @@
 #ifndef OF_OBJ
 #define OF_OBJ
 #endif
+#ifndef OF_OBJ2
+#define OF_OBJ2
+#endif
 #ifndef OF_BIN
 #define OF_BIN
 #endif
@@ -179,6 +185,9 @@
 #ifdef OF_NO_OBJ
 #undef OF_OBJ
 #endif
+#ifdef OF_NO_OBJ2
+#undef OF_OBJ2
+#endif
 #ifdef OF_NO_ELF32
 #undef OF_ELF32
 #endif
@@ -234,6 +243,7 @@ extern const struct ofmt of_elfx32;
 extern const struct ofmt of_elf64;
 extern const struct ofmt of_as86;
 extern const struct ofmt of_obj;
+extern const struct ofmt of_obj2;
 extern const struct ofmt of_win32;
 extern const struct ofmt of_win64;
 extern const struct ofmt of_ieee;
@@ -278,6 +288,9 @@ static const struct ofmt * const drivers[] = {
 #ifdef OF_OBJ
     &of_obj,
 #endif
+#ifdef OF_OBJ2
+    &of_obj2,
+#endif
 #ifdef OF_WIN32
     &of_win32,
 #endif
