@@ -80,7 +80,6 @@ sub genseqs($$$@) {
 
 sub startseq($$) {
     my ($codestr,$flags) = @_;
-    my $word;
     my @codes = ();
     my $c = $codestr;
     my($c0, $c1, $i);
@@ -488,7 +487,7 @@ if ( $output eq 'b') {
         print B " *";
         for ($j = 0; $j < 256; $j += 32) {
             print B " |" if ($j);
-            printf B " %3o:%4d", $i+$j, $bytecode_count[$i+$j];
+            printf B " %3o:%5d", $i+$j, $bytecode_count[$i+$j];
         }
         print B "\n";
     }
@@ -692,12 +691,14 @@ sub count_bytecodes(@) {
             $skip = 1;
         } elsif (($bc & ~013) == 0144) {
             $skip = 1;
-        } elsif ($bc == 0172 || $bc == 0173) {
+        } elsif ($bc >= 0171 && $bc <= 0173) {
             $skip = 1;
         } elsif (($bc & ~3) == 0260 || $bc == 0270) {   # VEX
             $skip = 2;
         } elsif (($bc & ~3) == 0240 || $bc == 0250) {   # EVEX
             $skip = 4;
+	} elsif (($bc & ~3) == 0304) {
+	    $skip = 2;
         } elsif ($bc == 0330) {
             $skip = 1;
         }
@@ -761,7 +762,7 @@ sub format_insn($$$$) {
 		    $opsz = $1 + 0;
                 }
 		if ($opp =~ s/0$//) {
-		    push(@oppx, 'rm_zero');
+		    push(@oppx, 'rn_zero');
 		    $iszero = 1;
 		    if ($opp !~ /reg/) {
 			$opp .= 'reg';
